/*  osweb CSS Definitions */
html, body {
  width: 100%;
  height: 100%; }

.d-flex {
  display: flex; }

.justify-content-center {
  justify-content: center; }

.flex-column {
  flex-direction: column; }

.stretch {
  flex: 1;
  height: 100%; }

.osweb-canvas-container {
  position: relative; }
  .osweb-canvas-container .aspectwrapper {
    /* Makes sure the osweb canvas retains it aspect ratio after a resize */
    display: inline-block;
    width: 100%;
    position: relative;
    text-align: center; }
    .osweb-canvas-container .aspectwrapper > canvas {
      background-color: black;
      position: relative;
      max-height: 100vh;
      max-width: 100vw;
      top: 0;
      bottom: 0;
      right: 0;
      left: 0;
      /* follow the parent's edges */ }

#osweb_form .cell {
  display: table-cell; }

.table,
.cell {
  height: 100%; }

#osweb_video {
  display: none; }

#dialogoverlay {
  display: none;
  top: 50%;
  left: 50%;
  transform: translate3d(-50%, -50%, 0);
  position: absolute;
  background: #FFF;
  opacity: .8;
  width: 100%;
  height: 100%;
  z-index: 10; }

#dialogbox {
  display: none;
  top: 50%;
  left: 50%;
  transform: translate3d(-50%, -50%, 0);
  position: absolute;
  background: #000;
  border-radius: 3px;
  width: 400px;
  z-index: 10; }
  #dialogbox > div {
    background: #FFF;
    margin: 2px; }
    #dialogbox > div > #dialogboxhead {
      background: #666;
      color: #CCC;
      font-size: 12px;
      padding: 10px; }
    #dialogbox > div > #dialogboxbody {
      background: #333;
      color: #FFF;
      padding: 20px; }
    #dialogbox > div > #dialogboxfoot {
      background: #666;
      padding: 10px;
      text-align: right; }

:full-screen canvas,
:-webkit-full-screen canvas,
:-ms-full-screen canvas,
:-moz-full-screen canvas {
  position: absolute;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
  margin: auto;
  width: 100%;
  height: 50%;
  background-color: black; }

input[type=checkbox].css-checkbox {
  position: absolute;
  z-index: -1000;
  left: -1000px;
  overflow: hidden;
  clip: rect(0 0 0 0);
  height: 1px;
  width: 1px;
  margin: -1px;
  padding: 0;
  border: 0; }

input[type=checkbox].css-checkbox + label.css-label {
  padding-left: 35px;
  height: 30px;
  display: inline-block;
  line-height: 30px;
  background-repeat: no-repeat;
  background-position: 0 0;
  font-size: 30px;
  vertical-align: middle;
  cursor: pointer; }

input[type=checkbox].css-checkbox:checked + label.css-label {
  background-position: 0 -30px; }

label.css-label {
  background-image: url(http://csscheckbox.com/checkboxes/u/csscheckbox_3f5bd3f7617a578c200aa742689c997e.png);
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -khtml-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none; }

/* Sticky footer styles
-------------------------------------------------- */
#footerbar {
  background-color: #f8f8f8;
  border-bottom: 1px solid #e7e7e7;
  border-top: 1px solid #e7e7e7;
  overflow: hidden;
  padding-top: 3px;
  width: 100%; }

#copyright {
  float: left; }

#licence {
  float: right; }


/*# sourceMappingURL=osweb.e2a1ecbc2a3f5bc08c82.bundle.css.map*/